﻿Imports PowerLanguage
Imports PowerLanguage.Function
Imports System
Imports System.Drawing

Namespace PowerLanguage.Indicator
    Public Class HPI
        Inherits IndicatorObject
        ' Methods
        Public Sub New(ByVal ctx As Object)
            MyBase.New(ctx)
            Me.m_onecent = 100
            Me.m_smoothingfactor = 0.133
            Me.m_alertlength = 14
            Me.m_colornormlength = 14
            Me.m_upcolor = Color.Yellow
            Me.m_dncolor = Color.Cyan
            Me.m_gridforegroundcolor = Color.FromKnownColor(KnownColor.Black)
        End Sub

        Protected Overrides Sub CalcBar()
            If (MyBase.Bars.CurrentBar = 1) Then
                Me.m_applicationtype.Value = DirectCast(MyBase.Environment.ApplicationCode, Integer)
            End If
            Me.m_hpivalue.Value = (Me.m_hpi1.Item(0) * 1E-05)
            Me.Plot1.Set(0, Me.m_hpivalue.Value)
            Me.Plot2.Set(0, 0)
            Me.m_colorlevel = Me.m_normgradientcolor1.Item(0)
            If (Me.m_applicationtype.Value = 1) Then
                Me.Plot1.Colors.Item(0) = Me.m_colorlevel
            ElseIf (Me.m_applicationtype.Value > 1) Then
                Me.Plot1.Colors.Item(0) = Me.m_gridforegroundcolor
                Me.Plot1.BGColor = Me.m_colorlevel
            End If
            If ((Me.m_lowestbar1.Item(0) = 0) AndAlso PublicFunctions.DoubleGreater(Me.m_lowestbar2.Item(0), 0)) Then
                MyBase.Alerts.Alert("Bullish divergence - new low not confirmed", New Object(0 - 1) {})
            ElseIf ((Me.m_highestbar1.Item(0) = 0) AndAlso PublicFunctions.DoubleGreater(Me.m_highestbar2.Item(0), 0)) Then
                MyBase.Alerts.Alert("Bearish divergence - new high not confirmed", New Object(0 - 1) {})
            End If
        End Sub

        Protected Overrides Sub Create()
            Me.m_hpi1 = New PowerLanguage.Function.HPI(Me)
            Me.m_normgradientcolor1 = New NormGradientColor(Me)
            Me.m_lowestbar1 = New LowestBar(Me)
            Me.m_lowestbar2 = New LowestBar(Me)
            Me.m_highestbar1 = New HighestBar(Me)
            Me.m_highestbar2 = New HighestBar(Me)
            Me.m_applicationtype = New VariableObject(Of Integer)(Me)
            Me.m_hpivalue = New VariableSeries(Of Double)(Me)
            Me.Plot1 = MyBase.AddPlot(New PlotAttributes("HPI", EPlotShapes.Line, Color.Yellow, Color.Empty, 0, 0, True))
            Me.Plot2 = MyBase.AddPlot(New PlotAttributes("ZeroLine", EPlotShapes.Line, Color.Green, Color.Empty, 0, 0, True))
        End Sub

        Protected Overrides Sub StartCalc()
            Me.m_hpi1.onecent = New Lambda(Of Double)( Function() Me.onecent)
            Me.m_hpi1.smfactor = New Lambda(Of Double)( Function() Me.smoothingfactor)
            Me.m_normgradientcolor1.dataseriesvalue = Me.m_hpivalue
            Me.m_normgradientcolor1.crosseszero = New ConstantObject(Of Boolean)(True)
            Me.m_normgradientcolor1.colornormlength = New Lambda(Of Integer)( Function() Me.colornormlength)
            Me.m_normgradientcolor1.upcolor = New Lambda(Of Color)(Function() Me.m_upcolor)
            Me.m_normgradientcolor1.dncolor = New Lambda(Of Color)(Function() Me.m_dncolor)
            Me.m_lowestbar1.price = MyBase.Bars.Close
            Me.m_lowestbar1.length = New Lambda(Of Integer)( Function() Me.alertlength)
            Me.m_lowestbar2.price = Me.m_hpivalue
            Me.m_lowestbar2.length = New Lambda(Of Integer)( Function() Me.alertlength)
            Me.m_highestbar1.price = MyBase.Bars.Close
            Me.m_highestbar1.length = New Lambda(Of Integer)( Function() Me.alertlength)
            Me.m_highestbar2.price = Me.m_hpivalue
            Me.m_highestbar2.length = New Lambda(Of Integer)( Function() Me.alertlength)
            Me.m_applicationtype.DefaultValue = 0
            Me.m_hpivalue.DefaultValue = 0
            Me.m_colorlevel = Color.Empty
        End Sub


        ' Properties
        <Input()> _
        Public Property alertlength As Integer
            Get
                Return Me.m_alertlength
            End Get
            Set(ByVal value As Integer)
                Me.m_alertlength = value
            End Set
        End Property

        <Input()> _
        Public Property colornormlength As Integer
            Get
                Return Me.m_colornormlength
            End Get
            Set(ByVal value As Integer)
                Me.m_colornormlength = value
            End Set
        End Property

        <Input()> _
        Public Property dncolor As KnownColor
            Get
                Return Me.m_dncolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_dncolor = Color.FromKnownColor(value)
            End Set
        End Property

        <Input()> _
        Public Property gridforegroundcolor As KnownColor
            Get
                Return Me.m_gridforegroundcolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_gridforegroundcolor = Color.FromKnownColor(value)
            End Set
        End Property

        <Input()> _
        Public Property onecent As Double
            Get
                Return Me.m_onecent
            End Get
            Set(ByVal value As Double)
                Me.m_onecent = value
            End Set
        End Property

        <Input()> _
        Public Property smoothingfactor As Double
            Get
                Return Me.m_smoothingfactor
            End Get
            Set(ByVal value As Double)
                Me.m_smoothingfactor = value
            End Set
        End Property

        <Input()> _
        Public Property upcolor As KnownColor
            Get
                Return Me.m_upcolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_upcolor = Color.FromKnownColor(value)
            End Set
        End Property


        ' Fields
        Private m_alertlength As Integer
        Private m_applicationtype As VariableObject(Of Integer)
        Private m_colorlevel As Color
        Private m_colornormlength As Integer
        Private m_dncolor As Color
        Private m_gridforegroundcolor As Color
        Private m_highestbar1 As HighestBar
        Private m_highestbar2 As HighestBar
        Private m_hpi1 As PowerLanguage.Function.HPI
        Private m_hpivalue As VariableSeries(Of Double)
        Private m_lowestbar1 As LowestBar
        Private m_lowestbar2 As LowestBar
        Private m_normgradientcolor1 As NormGradientColor
        Private m_onecent As Double
        Private m_smoothingfactor As Double
        Private m_upcolor As Color
        Private Plot1 As IPlotObject
        Private Plot2 As IPlotObject
    End Class
End Namespace

