﻿Imports PowerLanguage
Imports PowerLanguage.Function
Imports System
Imports System.Drawing

Namespace PowerLanguage.Indicator
    Public Class Money_Flow
        Inherits IndicatorObject
        ' Methods
        Public Sub New(ByVal ctx As Object)
            MyBase.New(ctx)
            Me.m_length = 14
            Me.m_alertlength = 14
            Me.m_oversold = 20
            Me.m_overbought = 80
            Me.m_overscolor = Color.Cyan
            Me.m_overbcolor = Color.Red
        End Sub

        Protected Overrides Sub CalcBar()
            Me.m_moneyflowval.Value = Me.m_moneyflow1.Item(0)
            Me.Plot1.Set(0, Me.m_moneyflowval.Value)
            Me.Plot2.Set(0, Me.overbought)
            Me.Plot3.Set(0, Me.oversold)
            If PublicFunctions.DoubleGreater(Me.m_moneyflowval.Value, Me.overbought) Then
                Me.Plot1.Colors.Item(0) = Me.m_overbcolor
            ElseIf PublicFunctions.DoubleLess(Me.m_moneyflowval.Value, Me.oversold) Then
                Me.Plot1.Colors.Item(0) = Me.m_overscolor
            End If
            If ((Me.m_lowestbar1.Item(0) = 0) AndAlso PublicFunctions.DoubleGreater(Me.m_lowestbar2.Item(0), 0)) Then
                MyBase.Alerts.Alert("Bullish divergence - new low not confirmed", New Object(0  - 1) {})
            ElseIf ((Me.m_highestbar1.Item(0) = 0) AndAlso PublicFunctions.DoubleGreater(Me.m_highestbar2.Item(0), 0)) Then
                MyBase.Alerts.Alert("Bearish divergence - new high not confirmed", New Object(0  - 1) {})
            End If
        End Sub

        Protected Overrides Sub Create()
            Me.m_moneyflow1 = New MoneyFlow(Me)
            Me.m_lowestbar1 = New LowestBar(Me)
            Me.m_lowestbar2 = New LowestBar(Me)
            Me.m_highestbar1 = New HighestBar(Me)
            Me.m_highestbar2 = New HighestBar(Me)
            Me.m_moneyflowval = New VariableSeries(Of Double)(Me)
            Me.Plot1 = MyBase.AddPlot(New PlotAttributes("MoneyFlow", EPlotShapes.Line, ColorTranslator.FromWin32(&HA4A0A0), Color.Empty, 0, 0, True))
            Me.Plot2 = MyBase.AddPlot(New PlotAttributes("OverBot", EPlotShapes.Line, Color.Green, Color.Empty, 0, 0, True))
            Me.Plot3 = MyBase.AddPlot(New PlotAttributes("OverSld", EPlotShapes.Line, Color.Green, Color.Empty, 0, 0, True))
        End Sub

        Protected Overrides Sub StartCalc()
            Me.m_moneyflow1.length = New Lambda(Of Integer)( Function() Me.length)
            Me.m_lowestbar1.price = MyBase.Bars.Close
            Me.m_lowestbar1.length = New Lambda(Of Integer)( Function() Me.alertlength)
            Me.m_lowestbar2.price = Me.m_moneyflowval
            Me.m_lowestbar2.length = New Lambda(Of Integer)( Function() Me.alertlength)
            Me.m_highestbar1.price = MyBase.Bars.Close
            Me.m_highestbar1.length = New Lambda(Of Integer)( Function() Me.alertlength)
            Me.m_highestbar2.price = Me.m_moneyflowval
            Me.m_highestbar2.length = New Lambda(Of Integer)( Function() Me.alertlength)
            Me.m_moneyflowval.DefaultValue = 0
        End Sub


        ' Properties
        <Input()> _
        Public Property alertlength As Integer
            Get
                Return Me.m_alertlength
            End Get
            Set(ByVal value As Integer)
                Me.m_alertlength = value
            End Set
        End Property

        <Input()> _
        Public Property length As Integer
            Get
                Return Me.m_length
            End Get
            Set(ByVal value As Integer)
                Me.m_length = value
            End Set
        End Property

        <Input()> _
        Public Property overbcolor As KnownColor
            Get
                Return Me.m_overbcolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_overbcolor = Color.FromKnownColor(value)
            End Set
        End Property

        <Input()> _
        Public Property overbought As Double
            Get
                Return Me.m_overbought
            End Get
            Set(ByVal value As Double)
                Me.m_overbought = value
            End Set
        End Property

        <Input()> _
        Public Property overscolor As KnownColor
            Get
                Return Me.m_overscolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_overscolor = Color.FromKnownColor(value)
            End Set
        End Property

        <Input()> _
        Public Property oversold As Double
            Get
                Return Me.m_oversold
            End Get
            Set(ByVal value As Double)
                Me.m_oversold = value
            End Set
        End Property


        ' Fields
        Private m_alertlength As Integer
        Private m_highestbar1 As HighestBar
        Private m_highestbar2 As HighestBar
        Private m_length As Integer
        Private m_lowestbar1 As LowestBar
        Private m_lowestbar2 As LowestBar
        Private m_moneyflow1 As MoneyFlow
        Private m_moneyflowval As VariableSeries(Of Double)
        Private m_overbcolor As Color
        Private m_overbought As Double
        Private m_overscolor As Color
        Private m_oversold As Double
        Private Plot1 As IPlotObject
        Private Plot2 As IPlotObject
        Private Plot3 As IPlotObject
    End Class
End Namespace

