﻿Imports PowerLanguage
Imports PowerLanguage.Function
Imports System
Imports System.Drawing
Imports System.Runtime.CompilerServices

Namespace PowerLanguage.Indicator
    <SameAsSymbol(True)> _
    Public Class Standard_Error_Bands
        Inherits IndicatorObject
        ' Methods
        Public Sub New(ByVal ctx As Object)
            MyBase.New(ctx)
            Me.m_linreglength = &H15
            Me.m_smoothinglength = 3
        End Sub

        Protected Overrides Sub CalcBar()
            Me.m_linregvalue.Value = Me.m_linearregvalue1.Item(0)
            Me.m_linregavg = PublicFunctions.Average(Me.m_linregvalue, Me.smoothinglength)
            Me.m_serroravg = PublicFunctions.Average(Me.m_stderror1, Me.smoothinglength)
            Me.m_lowerband.Value = (Me.m_linregavg - Me.m_serroravg)
            Me.m_upperband.Value = (Me.m_linregavg + Me.m_serroravg)
            If (((Me.displace >= 0) OrElse (MyBase.Bars.CurrentBar > Math.Abs(Me.displace))) AndAlso (MyBase.Bars.CurrentBar >= Me.smoothinglength)) Then
                Me.Plot1.Set(Me.displace, Me.m_upperband.Value)
                Me.Plot2.Set(Me.displace, Me.m_lowerband.Value)
                Me.Plot3.Set(Me.displace, Me.m_linregavg)
                If (Me.displace <= 0) Then
                    If Me.CrossesOver(Me.price, Me.m_lowerband) Then
                        MyBase.Alerts.Alert("Price crossing over lower price band", New Object(0  - 1) {})
                    ElseIf Me.CrossesUnder(Me.price, Me.m_upperband) Then
                        MyBase.Alerts.Alert("Price crossing under upper price band", New Object(0  - 1) {})
                    End If
                End If
            End If
        End Sub

        Protected Overrides Sub Create()
            Me.m_linearregvalue1 = New LinearRegValue(Me)
            Me.m_stderror1 = New StdError(Me)
            Me.m_linregvalue = New VariableSeries(Of Double)(Me)
            Me.m_lowerband = New VariableSeries(Of Double)(Me)
            Me.m_upperband = New VariableSeries(Of Double)(Me)
            Me.Plot1 = MyBase.AddPlot(New PlotAttributes("UpperBand", EPlotShapes.Line, Color.Yellow, Color.Empty, 0, 0, True))
            Me.Plot2 = MyBase.AddPlot(New PlotAttributes("LowerBand", EPlotShapes.Line, Color.Blue, Color.Empty, 0, 0, True))
            Me.Plot3 = MyBase.AddPlot(New PlotAttributes("MidLine", EPlotShapes.Line, Color.Gray, Color.Empty, 0, 0, True))
        End Sub

        Protected Overrides Sub StartCalc()
            Me.m_price = MyBase.Bars.Close
            Me.m_linearregvalue1.price = Me.price
            Me.m_linearregvalue1.length = New Lambda(Of Integer)( Function() Me.linreglength)
            Me.m_linearregvalue1.tgtbar = New ConstantObject(Of Integer)(0)
            Me.m_stderror1.price = Me.price
            Me.m_stderror1.length = New Lambda(Of Integer)( Function() Me.linreglength)
            Me.m_linregvalue.DefaultValue = 0
            Me.m_linregavg = 0
            Me.m_serroravg = 0
            Me.m_lowerband.DefaultValue = 0
            Me.m_upperband.DefaultValue = 0
        End Sub


        ' Properties
        Private m_displace As Integer
        <Input()> _
        Public Property displace As Integer
            Set(ByVal value As Integer)
                m_displace = value
            End Set
            Get
                Return Me.m_displace
            End Get
        End Property
        <Input()> _
        Public Property linreglength As Integer
            Get
                Return Me.m_linreglength
            End Get
            Set(ByVal value As Integer)
                Me.m_linreglength = value
            End Set
        End Property

        Private ReadOnly Property price As ISeries(Of Double)
            Get
                Return Me.m_price
            End Get
        End Property

        <Input()> _
        Public Property smoothinglength As Integer
            Get
                Return Me.m_smoothinglength
            End Get
            Set(ByVal value As Integer)
                Me.m_smoothinglength = value
            End Set
        End Property


        ' Fields
        Private m_linearregvalue1 As LinearRegValue
        Private m_linregavg As Double
        Private m_linreglength As Integer
        Private m_linregvalue As VariableSeries(Of Double)
        Private m_lowerband As VariableSeries(Of Double)
        Private m_price As ISeries(Of Double)
        Private m_serroravg As Double
        Private m_smoothinglength As Integer
        Private m_stderror1 As StdError
        Private m_upperband As VariableSeries(Of Double)
        Private Plot1 As IPlotObject
        Private Plot2 As IPlotObject
        Private Plot3 As IPlotObject
    End Class
End Namespace

