﻿Imports PowerLanguage
Imports PowerLanguage.Function
Imports System
Imports System.Drawing

Namespace PowerLanguage.Indicator
    <SameAsSymbol(True)> _
    Public Class Stochastic_SlowK
        Inherits IndicatorObject
        ' Methods
        Public Sub New(ByVal ctx As Object)
            MyBase.New(ctx)
            Me.m_length = 14
            Me.m_oversold = 20
            Me.m_overbought = 80
            Me.m_overscolor = Color.Cyan
            Me.m_overbcolor = Color.FromKnownColor(KnownColor.Blue)
        End Sub

        Protected Overrides Sub CalcBar()
            Me.m_myslowk = Me.m_slowkcustom1.Item(0)
            If PublicFunctions.DoubleLess(Me.m_myslowk, Me.oversold) Then
                Me.Plot1.Set(0, MyBase.Bars.High.Item(0), Me.m_overscolor)
                Me.Plot2.Set(0, MyBase.Bars.Low.Item(0), Me.m_overscolor)
                MyBase.Alerts.Alert("SlowK in oversold zone", New Object(0  - 1) {})
            ElseIf PublicFunctions.DoubleGreater(Me.m_myslowk, Me.overbought) Then
                Me.Plot1.Set(0, MyBase.Bars.High.Item(0), Me.m_overbcolor)
                Me.Plot2.Set(0, MyBase.Bars.Low.Item(0), Me.m_overbcolor)
                MyBase.Alerts.Alert("SlowK in overbought zone", New Object(0  - 1) {})
            Else
                Me.Plot1.Reset
            End If
        End Sub

        Protected Overrides Sub Create()
            Me.m_slowkcustom1 = New SlowKCustom(Me)
            Me.Plot1 = MyBase.AddPlot(New PlotAttributes("Plot1", EPlotShapes.BarHigh, Color.White, Color.Empty, 0, 0, True))
            Me.Plot2 = MyBase.AddPlot(New PlotAttributes("Plot2", EPlotShapes.BarLow, Color.White, Color.Empty, 0, 0, True))
        End Sub

        Protected Overrides Sub StartCalc()
            Me.m_priceh = MyBase.Bars.High
            Me.m_pricel = MyBase.Bars.Low
            Me.m_pricec = MyBase.Bars.Close
            Me.m_slowkcustom1.priceh = Me.priceh
            Me.m_slowkcustom1.pricel = Me.pricel
            Me.m_slowkcustom1.pricec = Me.pricec
            Me.m_slowkcustom1.stochlength = New Lambda(Of Integer)( Function() Me.length)
            Me.m_myslowk = 0
        End Sub


        ' Properties
        <Input> _
        Public Property length As Integer
            Get
                Return Me.m_length
            End Get
            Set(ByVal value As Integer)
                Me.m_length = value
            End Set
        End Property

        <Input> _
        Public Property overbcolor As KnownColor
            Get
                Return Me.m_overbcolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_overbcolor = Color.FromKnownColor(value)
            End Set
        End Property

        <Input> _
        Public Property overbought As Double
            Get
                Return Me.m_overbought
            End Get
            Set(ByVal value As Double)
                Me.m_overbought = value
            End Set
        End Property

        <Input> _
        Public Property overscolor As KnownColor
            Get
                Return Me.m_overscolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_overscolor = Color.FromKnownColor(value)
            End Set
        End Property

        <Input> _
        Public Property oversold As Double
            Get
                Return Me.m_oversold
            End Get
            Set(ByVal value As Double)
                Me.m_oversold = value
            End Set
        End Property

        Private ReadOnly Property pricec As ISeries(Of Double)
            Get
                Return Me.m_pricec
            End Get
        End Property

        Private ReadOnly Property priceh As ISeries(Of Double)
            Get
                Return Me.m_priceh
            End Get
        End Property

        Private ReadOnly Property pricel As ISeries(Of Double)
            Get
                Return Me.m_pricel
            End Get
        End Property


        ' Fields
        Private m_length As Integer
        Private m_myslowk As Double
        Private m_overbcolor As Color
        Private m_overbought As Double
        Private m_overscolor As Color
        Private m_oversold As Double
        Private m_pricec As ISeries(Of Double)
        Private m_priceh As ISeries(Of Double)
        Private m_pricel As ISeries(Of Double)
        Private m_slowkcustom1 As SlowKCustom
        Private Plot1 As IPlotObject
        Private Plot2 As IPlotObject
    End Class
End Namespace

