﻿Imports PowerLanguage
Imports PowerLanguage.Function
Imports System
Imports System.Drawing

Namespace PowerLanguage.Indicator
    Public Class Volume_Avg
        Inherits IndicatorObject
        ' Methods
        Public Sub New(ByVal ctx As Object)
            MyBase.New(ctx)
            Me.m_avglength = 50
            Me.m_alertpct = 50
            Me.m_upcolor = Color.Cyan
            Me.m_downcolor = Color.Red
        End Sub

        Protected Overrides Sub CalcBar()
            Dim resolution As EResolution = MyBase.Bars.Info.Resolution.Type
            If ((((EResolution.Day = resolution) OrElse (EResolution.Week = resolution)) OrElse ((EResolution.Month = resolution) OrElse (EResolution.Quarter = resolution))) OrElse (EResolution.Year = resolution)) Then
                Me.m_vvol.Value = MyBase.Bars.Volume.Item(0)
                Me.m_avgvvol.Value = Me.m_averagefc1.Item(0)
                Me.Plot1.Set(0, Me.m_vvol.Value)
                Me.Plot2.Set(0, Me.m_avgvvol.Value)
                If DirectCast(Me, IStudy).CrossesOver(DirectCast(Me.m_vvol, ISeries(Of Double)), CDbl((Me.m_avgvvol.Value * Me.m_alertfactor.Value))) Then
                    MyBase.Alerts.Alert(("Volume breaking through " & Me.m_alertstr & "% above its avg"), New Object(0 - 1) {})
                End If
            Else
                Me.m_tvol.Value = MyBase.Bars.Ticks.Item(0)
                Me.m_avgtvol.Value = Me.m_averagefc2.Item(0)
                Me.Plot1.Set(0, Me.m_tvol.Value)
                Me.Plot2.Set(0, Me.m_avgtvol.Value)
                If DirectCast(Me, IStudy).CrossesOver(DirectCast(Me.m_tvol, ISeries(Of Double)), CDbl((Me.m_avgtvol.Value * Me.m_alertfactor.Value))) Then
                    MyBase.Alerts.Alert(("Volume breaking through " & Me.m_alertstr & "% above its avg"), New Object(0 - 1) {})
                End If
            End If
            If PublicFunctions.DoubleGreater(MyBase.Bars.Close.Item(0), MyBase.Bars.Close.Item(1)) Then
                Me.Plot1.Colors.Item(0) = Me.m_upcolor
            ElseIf PublicFunctions.DoubleLess(MyBase.Bars.Close.Item(0), MyBase.Bars.Close.Item(1)) Then
                Me.Plot1.Colors.Item(0) = Me.m_downcolor
            End If
        End Sub

        Protected Overrides Sub Create()
            Me.m_averagefc1 = New AverageFC(Me)
            Me.m_averagefc2 = New AverageFC(Me)
            Me.m_vvol = New VariableSeries(Of Double)(Me)
            Me.m_avgvvol = New VariableSeries(Of Double)(Me)
            Me.m_tvol = New VariableSeries(Of Double)(Me)
            Me.m_avgtvol = New VariableSeries(Of Double)(Me)
            Me.m_alertfactor = New VariableSeries(Of Double)(Me)
            Me.Plot1 = MyBase.AddPlot(New PlotAttributes("Vol", EPlotShapes.Histogram, Color.White, Color.Empty, 0, 0, True))
            Me.Plot2 = MyBase.AddPlot(New PlotAttributes("VolAvg", EPlotShapes.Line, Color.Cyan, Color.Empty, 0, 0, True))
        End Sub

        Protected Overrides Sub StartCalc()
            Me.m_averagefc1.price = MyBase.Bars.Volume
            Me.m_averagefc1.length = New Lambda(Of Integer)( Function() Me.avglength)
            Me.m_averagefc2.price = MyBase.Bars.Ticks
            Me.m_averagefc2.length = New Lambda(Of Integer)( Function() Me.avglength)
            Me.m_vvol.DefaultValue = 0
            Me.m_avgvvol.DefaultValue = 0
            Me.m_tvol.DefaultValue = 0
            Me.m_avgtvol.DefaultValue = 0
            Me.m_alertfactor.DefaultValue = (1 + (Me.alertpct * 0.01))
            Me.m_alertstr = Math.Round(Me.alertpct, 2).ToString
        End Sub


        ' Properties
        <Input()> _
        Public Property alertpct As Double
            Get
                Return Me.m_alertpct
            End Get
            Set(ByVal value As Double)
                Me.m_alertpct = value
            End Set
        End Property

        <Input()> _
        Public Property avglength As Integer
            Get
                Return Me.m_avglength
            End Get
            Set(ByVal value As Integer)
                Me.m_avglength = value
            End Set
        End Property

        <Input()> _
        Public Property downcolor As KnownColor
            Get
                Return Me.m_downcolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_downcolor = Color.FromKnownColor(value)
            End Set
        End Property

        <Input()> _
        Public Property upcolor As KnownColor
            Get
                Return Me.m_upcolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_upcolor = Color.FromKnownColor(value)
            End Set
        End Property


        ' Fields
        Private m_alertfactor As VariableSeries(Of Double)
        Private m_alertpct As Double
        Private m_alertstr As String
        Private m_averagefc1 As AverageFC
        Private m_averagefc2 As AverageFC
        Private m_avglength As Integer
        Private m_avgtvol As VariableSeries(Of Double)
        Private m_avgvvol As VariableSeries(Of Double)
        Private m_downcolor As Color
        Private m_tvol As VariableSeries(Of Double)
        Private m_upcolor As Color
        Private m_vvol As VariableSeries(Of Double)
        Private Plot1 As IPlotObject
        Private Plot2 As IPlotObject
    End Class
End Namespace

