﻿Imports PowerLanguage
Imports PowerLanguage.Function
Imports System
Imports System.Drawing

Namespace PowerLanguage.Indicator
    Public Class Volume_Ratio
        Inherits IndicatorObject
        ' Methods
        Public Sub New(ByVal ctx As Object)
            MyBase.New(ctx)
            Me.m_avglength = 14
            Me.m_upvolalertpct = 5
            Me.m_dnvolalertpct = -5
            Me.m_colornormlength = 14
            Me.m_upcolor = Color.Yellow
            Me.m_dncolor = Color.Cyan
        End Sub

        Protected Overrides Sub CalcBar()
            Dim resolution As EResolution = MyBase.Bars.Info.Resolution.Type
            If ((((resolution < EResolution.Day) OrElse (EResolution.Volume = resolution)) OrElse (EResolution.Second = resolution)) OrElse (resolution > EResolution.Quarter)) Then
                Me.m_totalticks = (MyBase.Bars.UpTicks.Item(0) + MyBase.Bars.DownTicks.Item(0))
                If PublicFunctions.DoubleGreater(Me.m_totalticks, 0) Then
                    Me.m_volratio.Value = ((100 * (MyBase.Bars.UpTicks.Item(0) - MyBase.Bars.DownTicks.Item(0))) / Me.m_totalticks)
                Else
                    Me.m_volratio.Value = 0
                End If
                Me.m_volratioavg.Value = Me.m_xaverage1.Item(0)
                Me.Plot1.Set(0, Me.m_volratioavg.Value)
                Me.Plot2.Set(0, 0)
                Me.m_colorlevel = Me.m_normgradientcolor1.Item(0)
                Me.Plot1.Colors.Item(0) = Me.m_colorlevel
                If Me.CrossesOver(Me.m_volratioavg, New Lambda(Of Double)( Function() Me.upvolalertpct)) Then
                    MyBase.Alerts.Alert("UpVol alert", New Object(0 - 1) {})
                ElseIf Me.CrossesUnder(Me.m_volratioavg, New Lambda(Of Double)( Function() Me.dnvolalertpct)) Then
                    MyBase.Alerts.Alert("DnVol alert", New Object(0 - 1) {})
                End If
            End If
        End Sub

        Protected Overrides Sub Create()
            Me.m_xaverage1 = New XAverage(Me)
            Me.m_normgradientcolor1 = New NormGradientColor(Me)
            Me.m_volratio = New VariableSeries(Of Double)(Me)
            Me.m_volratioavg = New VariableSeries(Of Double)(Me)
            Me.Plot1 = MyBase.AddPlot(New PlotAttributes("VolRatioAvg", EPlotShapes.Line, Color.Blue, Color.Empty, 0, 0, True))
            Me.Plot2 = MyBase.AddPlot(New PlotAttributes("ZeroLine", EPlotShapes.Line, Color.Green, Color.Empty, 7, 7, True))
        End Sub

        Protected Overrides Sub StartCalc()
            Me.m_xaverage1.Price = Me.m_volratio
            Me.m_xaverage1.Length = Me.avglength
            Me.m_normgradientcolor1.dataseriesvalue = Me.m_volratioavg
            Me.m_normgradientcolor1.crosseszero = New ConstantObject(Of Boolean)(True)
            Me.m_normgradientcolor1.colornormlength = New Lambda(Of Integer)( Function() Me.colornormlength)
            Me.m_normgradientcolor1.upcolor = New Lambda(Of Color)( Function() Me.m_upcolor)
            Me.m_normgradientcolor1.dncolor = New Lambda(Of Color)( Function() Me.m_dncolor)
            Me.m_totalticks = 0
            Me.m_volratio.DefaultValue = 0
            Me.m_volratioavg.DefaultValue = 0
            Me.m_colorlevel = Color.Empty
        End Sub


        ' Properties
        <Input()> _
        Public Property avglength As Integer
            Get
                Return Me.m_avglength
            End Get
            Set(ByVal value As Integer)
                Me.m_avglength = value
            End Set
        End Property

        <Input()> _
        Public Property colornormlength As Integer
            Get
                Return Me.m_colornormlength
            End Get
            Set(ByVal value As Integer)
                Me.m_colornormlength = value
            End Set
        End Property

        <Input()> _
        Public Property dncolor As KnownColor
            Get
                Return Me.m_dncolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_dncolor = Color.FromKnownColor(value)
            End Set
        End Property

        <Input()> _
        Public Property dnvolalertpct As Double
            Get
                Return Me.m_dnvolalertpct
            End Get
            Set(ByVal value As Double)
                Me.m_dnvolalertpct = value
            End Set
        End Property

        <Input()> _
        Public Property upcolor As KnownColor
            Get
                Return Me.m_upcolor.ToKnownColor
            End Get
            Set(ByVal value As KnownColor)
                Me.m_upcolor = Color.FromKnownColor(value)
            End Set
        End Property

        <Input()> _
        Public Property upvolalertpct As Double
            Get
                Return Me.m_upvolalertpct
            End Get
            Set(ByVal value As Double)
                Me.m_upvolalertpct = value
            End Set
        End Property


        ' Fields
        Private m_avglength As Integer
        Private m_colorlevel As Color
        Private m_colornormlength As Integer
        Private m_dncolor As Color
        Private m_dnvolalertpct As Double
        Private m_normgradientcolor1 As NormGradientColor
        Private m_totalticks As Double
        Private m_upcolor As Color
        Private m_upvolalertpct As Double
        Private m_volratio As VariableSeries(Of Double)
        Private m_volratioavg As VariableSeries(Of Double)
        Private m_xaverage1 As XAverage
        Private Plot1 As IPlotObject
        Private Plot2 As IPlotObject
    End Class
End Namespace

